@@ -2,7 +2,7 @@ package Date::ICal;
 use strict;
 
 use vars qw($VERSION $localzone $localoffset @months @leapmonths);
-$VERSION = (qw'$Revision: 1.53 $')[1];
+$VERSION = (qw'$Revision: 1.55 $')[1];
 use Carp;
 use Time::Local;
 use Date::Leapyear qw();
@@ -523,24 +523,49 @@ sub add {
         $seconds += $args{min}  * 60       if defined $args{min};
         $seconds += $args{hour} * 60 * 60  if defined $args{hour};
 
-	$days = 0;
+        $days = 0;
         $days += $args{day}       if defined $args{day};
         $days += $args{week} * 7  if defined $args{week};
 
-	if ($args{month}) {
-	    my @months = months($self->year);
-	    my $start = $months[$self->month - 1];
-	    my $end = $self->month + $args{month};
-	    my $add = 0;
-	    if ($end > 12) {
-		$end -= 12;
-		$add = $months[12];
-		@months = months($self->year + 1);
-	    }
-	    $end = $months[$end - 1] + $add;
+        if ($args{month}) {
+            my @months = months($self->year);
+            my $start = $months[$self->month - 1];
+            my $end = $self->month + $args{month};
+            my $add = 0;
+            if ($end > 12) {
+                $end -= 12;
+                $add = $months[12];
+                @months = months($self->year + 1);
+            }
+            $end = $months[$end - 1] + $add;
+
+            $days += $end - $start;
+        }
+
+        if ($args{year}) {
+            foreach my $year ($self->year .. $self->year + $args{year} - 1) {
+                my $leap = Date::Leapyear::isleap($year);
+                $days += 365 + $leap;
+            }
+
+            # Remove a day if the year that we started in was leap, but
+            # we started *after* the leap day
+            if (Date::Leapyear::isleap($self->year) &&
+                days_this_year($self->day, $self->month, $self->year) >=
+                60) {
+                $days--;
+            }
+
+            # Add a day is the year we finish in is leap, and we end
+            # *after* the leap day
+            if (Date::Leapyear::isleap($self->year + $args{year}) &&
+                days_this_year($self->day, $self->month, $self->year +
+                $args{year}) >= 60) {
+                $days++;
+            }
+        }
+
 
-	    $days += $end - $start;
-	}
     }
 
     # ick, we really don't want this.
@@ -812,6 +837,8 @@ sub greg2jd {
 
 =head2 days_this_year
 
+  $yday = Date::ICal::days_this_year($day, $month, $year);
+
 Returns the number of days so far this year. Analogous to the yday
 attribute of gmtime (or localtime) except that it works outside of the
 epoch.
@@ -819,7 +846,7 @@ epoch.
 =cut
 
 sub days_this_year {
-    my ($y,$m,$d) = @_;
+    my ($d,$m,$y) = @_;
     my $jd = greg2jd($y,$m,$d);
     my $janone = greg2jd($y,1,1);
     return $jd - $janone;
@@ -989,6 +1016,13 @@ Net::ICal
 =head1 CVS History
 
   $Log: ICal.pm,v $
+  Revision 1.55  2001/11/28 02:00:16  rbowen
+  Able to add n years to a date via the add method. Tests to match.
+
+  Revision 1.54  2001/11/24 18:57:37  rbowen
+  Oops. I reversed the order of the argument list when I added this
+  function back in, thereby breaking all code that was calling it.
+
   Revision 1.53  2001/11/24 16:25:10  rbowen
   Since _seconds_from_offset returns a number, not a string, we only need
   the sign if it is negative. Resolves some test failures that I was
